==> rsMove_0.2.4.tar.gz: rsMove/R/moveReduce.R <==
#' @title moveReduce
#'
#' @description Pixel based summary of movement data that preserves periodic movements.
#' @param x Object of class \emph{SpatialPoints} or \emph{SpatialPointsDataFrame}.
#' @param z Object of class \emph{Date}, \emph{POSIXlt} or \emph{POSIXct} with \emph{x} observation dates.
#' @param y Object of class \emph{RasterLayer}, \emph{RasterStack} or \emph{RasterBrick}.
#' @param derive.raster Should a raster with the total time per pixel be provided?
#' @importFrom raster crs cellFromXY rasterize
#' @importFrom sp SpatialPointsDataFrame
#' @seealso \code{\link{sampleMove}} \code{\link{moveSeg}}
#' @return A \emph{list} object.
#' @details {Reduces a set of input samples (\emph{x}) based on their corresponding pixel coordinates
#' within a reference raster (\emph{y}). Using this data, the function identifies temporal segments
#' corresponding to groups of consecutive samples found within the same pixel. In this process, revisits
#' to recorded pixels are preserved. Once the segments are identified, the function derives mean x and y
#' coordinates for each of them and evaluates the time spent within each pixel. The function reports on
#' the start and end timestamps, the mean timestamp and the elapsed time. The output of the function
#' consists of:
#' \itemize{
#' \item{\emph{r.shp} - Shapefile with reduced sample set and its corresponding temporal information.}
#' \item{\emph{total.time} - Raster showing the total time spent at each pixel (if \emph{derive.raster} is TRUE).}
#' \item{\emph{indices} - Indices for each sample in \emph{x} showing which samples were aggregated.}}
#'
#' }
#' @examples {
#'
#'  require(raster)
#'
#'  # read raster data
#'  r <- raster(system.file('extdata', '2013-07-16_ndvi.tif', package="rsMove"))
#'
#'  # read movement data
#'  data(shortMove)
#'
#'  # observation time
#'  z <- strptime(paste0(shortMove@data$date, ' ', shortMove@data$time),
#'  format="%Y/%m/%d %H:%M:%S")
#'
#'  # reduce amount of samples
#'  move.reduce <- moveReduce(shortMove, r, z)
#'
#' }
#' @export

#----------------------------------------------------------------------------------------------------------#

moveReduce <- function(x, y, z, derive.raster=FALSE) {

#----------------------------------------------------------------------------------------------------------#
# 1. check input variables
#----------------------------------------------------------------------------------------------------------#

  # samples
  if (!exists('x')) {stop('"x" is missing')}
  if (!class(x)[1]%in%c('SpatialPoints', 'SpatialPointsDataFrame')) {stop('"x" is not of a valid class')}
  rProj <- crs(x) # output projection

  # sample dates
  if (!is.null(z)) {
    if (!class(z)[1]%in%c('Date', 'POSIXct', 'POSIXlt')) {stop('"z" is nof of a valid class')}
    if (length(z)!=length(x)) {stop('errorr: "x" and "z" have different lengths')}}

  # environmental data
  if (!class(y)[1]%in%c('RasterLayer', 'RasterStack', 'RasterBrick')) {
    stop('"y" is not a valid raster object')}
  if (crs(x)@projargs!=crs(y)@projargs) {stop('"x" and "edata" have different projections')}

#----------------------------------------------------------------------------------------------------------#
# 2. identify segments
#----------------------------------------------------------------------------------------------------------#

  # convert x to single pixels
  os <- order(z)
  x <- x[os,]
  z <- z[os]
  sp <- cellFromXY(y, x@coords)

  rm(os)

  # search for segments and return sample indices
  pd <- rle(sp)$lengths
  sg <- vector('numeric', length(sp))
  for (p in 1:length(pd)) {sg[(sum(pd[0:(p-1)])+1):sum(pd[1:p])] <- p}

  rm(pd)

  # estimate
  df <- do.call(rbind, lapply(1:max(sg), function(s) {
    ind <- which(sg==s)
    mx <- median(x@coords[ind,1])
    my <- median(x@coords[ind,2])
    s.time <- z[ind[1]]
    e.time <- z[ind[length(ind)]]
    d.time <- as.numeric(difftime(e.time, s.time, units='mins'))
    return(data.frame(x=mx, y=my, start.time=s.time, end.time=e.time, diff.time=d.time, segment.id=s))}))
  colnames(df) <- c("x", "y", "Timeststamp (start)", "Timeststamp (end)",
                    "Elapsed time (minutes)", "Segment ID")

  # build statistic shapefile
  r.shp <- SpatialPointsDataFrame(df[,1:2], df, proj4string=crs(x))

#----------------------------------------------------------------------------------------------------------#
# 3. derive single raster
#----------------------------------------------------------------------------------------------------------#

  if (derive.raster) {

    # find cell positions of reduced sample set
    sp <- cellFromXY(y, df[,c("x", "y")])
    up <- unique(sp)

    # estimate time sum per cell
    t.sum <- sapply(up, function(p) {sum(df$'Elapsed time (minutes)'[which(sp==p)], na.rm=TRUE)})

    # build raster
    t.sum.r <- rasterize(xyFromCell(y, up), crop(y, extent(x)), t.sum)

  } else {t.sum.r <- NULL}

#----------------------------------------------------------------------------------------------------------#
# 4. build output
#----------------------------------------------------------------------------------------------------------#

  return(list(points=r.shp, total.time=t.sum.r, indices=sg))

}

==> rsMove_0.2.5.tar.gz: rsMove/R/moveReduce.R <==
#' @title moveReduce
#'
#' @description Pixel based summary of movement data that preserves periodic movements.
#' @param x Object of class \emph{SpatialPoints} or \emph{SpatialPointsDataFrame}.
#' @param z Object of class \emph{Date}, \emph{POSIXlt} or \emph{POSIXct} with the observation time of \emph{x}.
#' @param y Object of class \emph{RasterLayer}, \emph{RasterStack} or \emph{RasterBrick}.
#' @param derive.raster Should a \emph{RasterLayer} with the total time per pixel be provided?
#' @importFrom raster crs cellFromXY rasterize
#' @importFrom sp SpatialPointsDataFrame
#' @seealso \code{\link{sampleMove}} \code{\link{moveSeg}}
#' @return A \emph{list} object.
#' @details {Translates (\emph{x}) into pixel coordinates within a reference raster (\emph{y}). The function
#' identifies temporal segments corresponding to groups of consecutive observations within the same pixel. In this process, revisits
#' to recorded pixels are preserved. Once the segments are identified, the function derives mean x and y
#' coordinates for each of them and evaluates the time spent within each pixel. The function reports on
#' the start and end timestamps, the mean timestamp and the elapsed time. The output of the function
#' consists of:
#' \itemize{
#' \item{\emph{points} - Shapefile with reduced sample set and its corresponding temporal information.}
#' \item{\emph{total.time} - Raster showing the total time spent at each pixel (if \emph{derive.raster} is TRUE).}
#' \item{\emph{indices} - Indices for each sample in \emph{x} showing which samples were aggregated.}}
#'
#' }
#' @examples {
#'
#'  require(raster)
#'
#'  # read raster data
#'  r <- raster(system.file('extdata', '2013-07-16_ndvi.tif', package="rsMove"))
#'
#'  # read movement data
#'  data(shortMove)
#'
#'  # observation time
#'  z <- strptime(paste0(shortMove@data$date, ' ', shortMove@data$time),
#'  format="%Y/%m/%d %H:%M:%S")
#'
#'  # reduce amount of samples
#'  move.reduce <- moveReduce(shortMove, r, z)
#'
#' }
#' @export

#----------------------------------------------------------------------------------------------------------#

moveReduce <- function(x, y, z, derive.raster=FALSE) {

#----------------------------------------------------------------------------------------------------------#
# 1. check input variables
#----------------------------------------------------------------------------------------------------------#

  # samples
  if (!class(x)[1]%in%c('SpatialPoints', 'SpatialPointsDataFrame')) {stop('"x" is not of a valid class')}

  # sample dates
  if (!class(z)[1]%in%c('Date', 'POSIXct', 'POSIXlt')) {stop('"z" is nof of a valid class')}
  if (length(z)!=length(x)) {stop('errorr: "x" and "z" have different lengths')}

  # environmental data
  if (!class(y)[1]%in%c('RasterLayer', 'RasterStack', 'RasterBrick')) {
    stop('"y" is not a valid raster object')}
  if (crs(x)@projargs!=crs(y)@projargs) {stop('"x" and "edata" have different projections')}

#----------------------------------------------------------------------------------------------------------#
# 2. identify segments
#----------------------------------------------------------------------------------------------------------#

  # convert x to single pixels
  os <- order(z)
  x <- x[os,]
  z <- z[os]
  sp <- cellFromXY(y, x@coords)

  rm(os)

  # search for segments and return sample indices
  pd <- rle(sp)$lengths
  sg <- vector('numeric', length(sp))
  for (p in 1:length(pd)) {sg[(sum(pd[0:(p-1)])+1):sum(pd[1:p])] <- p}

  rm(pd)

  # estimate
  df <- do.call(rbind, lapply(1:max(sg), function(s) {
    ind <- which(sg==s)
    mx <- median(x@coords[ind,1])
    my <- median(x@coords[ind,2])
    s.time <- z[ind[1]]
    e.time <- z[ind[length(ind)]]
    d.time <- as.numeric(difftime(e.time, s.time, units='mins'))
    return(data.frame(x=mx, y=my, start.time=s.time, end.time=e.time, diff.time=d.time, segment.id=s))}))
  colnames(df) <- c("x", "y", "Timeststamp (start)", "Timeststamp (end)", "Elapsed time (minutes)", "Segment ID")

  # build statistic shapefile
  r.shp <- SpatialPointsDataFrame(df[,1:2], df, proj4string=crs(x))

#----------------------------------------------------------------------------------------------------------#
# 3. derive single raster
#----------------------------------------------------------------------------------------------------------#

  if (derive.raster) {

    # find cell positions of reduced sample set
    sp <- cellFromXY(y, df[,c("x", "y")])
    up <- unique(sp)

    # estimate time sum per cell
    t.sum <- sapply(up, function(p) {sum(df$'Elapsed time (minutes)'[which(sp==p)], na.rm=TRUE)})

    # build raster
    t.sum.r <- rasterize(xyFromCell(y, up), y, t.sum)

  } else {t.sum.r <- NULL}

#----------------------------------------------------------------------------------------------------------#
# 4. build output
#----------------------------------------------------------------------------------------------------------#

  return(list(points=r.shp, total.time=t.sum.r, indices=sg))

}
